# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from __future__ import generators

__version__ = "0.5d"
import sys
from nevow import compy
from nevow.util import namedAny as na


def getAOI(fObj):
    for line in fObj:
        line = line.strip()
        if line and not line.startswith('#'):
            yield line.split()


# Python2.2 has a stupidity where instance methods have name
# '__builtin__.instance method' instead of '__builtin__.instancemethod'
# Workaround this error.
def load(S):
    for (a, o, i) in getAOI(S.split('\n')):
        if o == '__builtin__.instancemethod' and sys.version_info < (2,3):
            compy.registerAdapter(a, '__builtin__.instance method', i)
        else:
            compy.registerAdapter(a, o, i)

basic_adapters = """
nevow.formless.Group                   nevow.formless.MetaTypedInterface        nevow.formless.ITyped

nevow.accessors.DictionaryContainer    __builtin__.dict                         nevow.inevow.IContainer
nevow.accessors.ListContainer          __builtin__.list                         nevow.inevow.IContainer
nevow.accessors.ListContainer          __builtin__.tuple                        nevow.inevow.IContainer

nevow.accessors.FunctionAccessor       __builtin__.function                     nevow.inevow.IGettable
nevow.accessors.FunctionAccessor       __builtin__.method                       nevow.inevow.IGettable
nevow.accessors.FunctionAccessor       __builtin__.instancemethod               nevow.inevow.IGettable
nevow.accessors.DirectiveAccessor      nevow.stan.directive                     nevow.inevow.IGettable

nevow.flat.flatmdom.MicroDomDocumentSerializer          twisted.web.microdom.Document           nevow.inevow.ISerializable
nevow.flat.flatmdom.MicroDomTextSerializer              twisted.web.microdom.Text               nevow.inevow.ISerializable
nevow.flat.flatmdom.MicroDomCommentSerializer           twisted.web.microdom.Comment            nevow.inevow.ISerializable
nevow.flat.flatmdom.MicroDomElementSerializer           twisted.web.microdom.Element            nevow.inevow.ISerializable
nevow.flat.flatmdom.MicroDomEntityReferenceSerializer   twisted.web.microdom.EntityReference    nevow.inevow.ISerializable

nevow.flat.flatstan.ProtoSerializer               nevow.stan.Proto                  nevow.inevow.ISerializable
nevow.flat.flatstan.TagSerializer                 nevow.stan.Tag                    nevow.inevow.ISerializable
nevow.flat.flatstan.CommentSerializer             nevow.stan.Comment                nevow.inevow.ISerializable
nevow.flat.flatstan.XmlSerializer                 nevow.stan.xml                    nevow.inevow.ISerializable
nevow.flat.flatstan.StringSerializer              __builtin__.str                   nevow.inevow.ISerializable
nevow.flat.flatstan.UTF8Serializer                __builtin__.unicode               nevow.inevow.ISerializable
nevow.flat.flatstan.NoneWarningSerializer         __builtin__.NoneType              nevow.inevow.ISerializable
nevow.flat.flatstan.StringCastSerializer          __builtin__.int                   nevow.inevow.ISerializable
nevow.flat.flatstan.StringCastSerializer          __builtin__.float                 nevow.inevow.ISerializable
nevow.flat.flatstan.StringCastSerializer          __builtin__.long                  nevow.inevow.ISerializable
nevow.flat.flatstan.StringCastSerializer          __builtin__.bool                  nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer                __builtin__.list                  nevow.inevow.ISerializable
nevow.flat.flatstan.StringCastSerializer          __builtin__.dict                  nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer                __builtin__.tuple                 nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer                __builtin__.generator             nevow.inevow.ISerializable
nevow.flat.flatstan.FunctionSerializer            __builtin__.function              nevow.inevow.ISerializable
nevow.flat.flatstan.FunctionSerializer            __builtin__.type                  nevow.inevow.ISerializable
nevow.flat.flatstan.MethodSerializer              __builtin__.instancemethod        nevow.inevow.ISerializable
nevow.flat.flatstan.CallableInstanceSerializer    nevow.inevow.IRendererFactory     nevow.inevow.ISerializable
nevow.flat.flatstan.DirectiveSerializer           nevow.stan.directive              nevow.inevow.ISerializable
nevow.flat.flatstan.SlotSerializer                nevow.stan.slot                   nevow.inevow.ISerializable
nevow.flat.flatstan.ContextSerializer             nevow.context.WovenContext        nevow.inevow.ISerializable
nevow.flat.flatstan.DeferredSerializer            twisted.internet.defer.Deferred   nevow.inevow.ISerializable

nevow.url.URLOverlaySerializer            nevow.url.URLOverlay     nevow.inevow.ISerializable

    # Itertools uses special types
nevow.flat.flatstan.ListSerializer  itertools.chain        nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.count        nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.cycle        nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.dropwhile    nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.ifilter      nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.ifilterfalse nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.imap         nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.islice       nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.izip         nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.repeat       nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.starmap      nevow.inevow.ISerializable
nevow.flat.flatstan.ListSerializer  itertools.takewhile    nevow.inevow.ISerializable

    # object
    # list
    # table
    # request
    # compound

nevow.freeform.PropertyBindingRenderer  nevow.formless.Property         nevow.freeform.IBindingRenderer
nevow.freeform.MethodBindingRenderer    nevow.formless.MethodBinding    nevow.freeform.IBindingRenderer
nevow.freeform.ArgumentBindingRenderer  nevow.formless.Argument         nevow.freeform.IBindingRenderer
nevow.freeform.GroupBindingRenderer     nevow.formless.GroupBinding     nevow.freeform.IBindingRenderer

    #

nevow.freeform.StringRenderer       nevow.formless.String       nevow.freeform.ITypedRenderer
nevow.freeform.StringRenderer       nevow.formless.Integer      nevow.freeform.ITypedRenderer
nevow.freeform.StringRenderer       nevow.formless.Directory    nevow.freeform.ITypedRenderer
nevow.freeform.PasswordRenderer     nevow.formless.Password     nevow.freeform.ITypedRenderer
nevow.freeform.TextRenderer         nevow.formless.Text         nevow.freeform.ITypedRenderer
nevow.freeform.BooleanRenderer      nevow.formless.Boolean      nevow.freeform.ITypedRenderer
nevow.freeform.ChoiceRenderer       nevow.formless.Choice       nevow.freeform.ITypedRenderer
nevow.freeform.ButtonRenderer       nevow.formless.Button       nevow.freeform.ITypedRenderer
nevow.freeform.ObjectRenderer       nevow.formless.Object       nevow.freeform.ITypedRenderer
nevow.freeform.RequestRenderer      nevow.formless.Request      nevow.freeform.ITypedRenderer
nevow.freeform.ListRenderer         nevow.formless.List         nevow.freeform.ITypedRenderer
nevow.freeform.DictionaryRenderer   nevow.formless.Dictionary   nevow.freeform.ITypedRenderer
nevow.freeform.FileUploadRenderer   nevow.formless.FileUpload   nevow.freeform.ITypedRenderer

    #

nevow.freeform.ActionRenderer       nevow.formless.MethodBinding        nevow.freeform.IActionRenderer

    #

nevow.freeform.GroupBindingValidator    nevow.formless.GroupBinding     nevow.inevow.IInputValidator
nevow.freeform.MethodBindingValidator   nevow.formless.MethodBinding    nevow.inevow.IInputValidator
nevow.freeform.PropertyBindingValidator nevow.formless.Property         nevow.inevow.IInputValidator
nevow.freeform.TypedValidator           nevow.formless.ITyped           nevow.inevow.IInputValidator
nevow.freeform.PasswordValidator        nevow.formless.Password         nevow.inevow.IInputValidator
nevow.freeform.RequestValidator         nevow.formless.Request          nevow.inevow.IInputValidator
nevow.freeform.ListValidator            nevow.formless.List             nevow.inevow.IInputValidator
nevow.freeform.UploadValidator          nevow.formless.FileUpload       nevow.inevow.IInputValidator


nevow.freeform.FormDefaults     nevow.appserver.NevowRequest                nevow.freeform.IFormDefaults
nevow.freeform.FormDefaults     nevow.testutil.FakeRequest                  nevow.freeform.IFormDefaults
nevow.freeform.FormDefaults     nevow.testutil.FakeSession                  nevow.freeform.IFormDefaults
nevow.freeform.FormDefaults     twisted.web.server.Session                  nevow.freeform.IFormDefaults
nevow.freeform.FormDefaults     twisted.web.woven.guard.GuardSession        nevow.freeform.IFormDefaults
nevow.freeform.FormDefaults     nevow.guard.GuardSession                    nevow.freeform.IFormDefaults

nevow.freeform.FormErrors       twisted.web.server.Session               nevow.freeform.IFormErrors
nevow.freeform.FormErrors       twisted.web.woven.guard.GuardSession     nevow.freeform.IFormErrors
nevow.freeform.FormErrors       nevow.guard.GuardSession                 nevow.freeform.IFormErrors
nevow.freeform.FormErrors       nevow.testutil.FakeSession               nevow.freeform.IFormErrors

nevow.appserver.OldResourceAdapter                  twisted.web.resource.IResource      nevow.inevow.IResource
nevow.freeform.DefaultTypedInterfaceToIResource     nevow.formless.TypedInterface       nevow.inevow.IResource

nevow.formless.ListConfigurable __builtin__.list  nevow.formless.IConfigurable

nevow.rend.DocFactorySerializer nevow.inevow.IDocFactory  nevow.inevow.ISerializable
"""

load(basic_adapters)
