from xml.sax import make_parser, handler
import xml as pyxml

from nevow.stan import xml, Tag, directive

## Require PyXML 0.8.2 or later, or, if PyXML isn't installed
## python2.3 or later, because that includes approximately the
## same code (but doesn't share a version number *!@#$@!@#)

try:
    ## pyxml package has a version_info attribute
    bad_version = pyxml.version_info < (0,8,2)
except:
    ## we're using core python xml library
    import sys
    bad_version = sys.version_info < (2,3)

class ToStan(handler.ContentHandler, handler.EntityResolver):
    directiveMapping = {
        'render': 'render',
        'data': 'data',
    }
    attributeList = [
        'pattern', 'key',
    ]

    def resolveEntity(self, publicId, systemId):
        ## This doesn't seem to get called, which is good.
        raise Exception("resolveEntity should not be called. We don't use external DTDs.")

    def skippedEntity(self, name):
        self.current.append(xml("&%s;"%name))
        
    def startDocument(self):
        self.document = []
#        self.document = [xml('<?xml version="1.0"?>\n')]
        self.current = self.document
        self.stack = []

    def endDocument(self):
        pass

    def processingInstruction(self, target, data):
        self.current.append(xml("<?%s %s?>\n" % (target, data)))

    def startElement(self, name, attrs):
        if name.startswith('nevow:'):
            _, name = name.split(':')
            if name == 'invisible':
                name = ''
            elif name == 'slot':
                el = slot(element.attributes['name'])
                self.current.append(el)
                fakeEl = Tag('') # eat children
                self.current = fakeEl
                self.stack.append(self.current)
        
        attrs = dict(attrs)
        specials = {}
        attributes = self.attributeList
        directives = self.directiveMapping
        for k, v in attrs.items():
            if not k.startswith('nevow:'):
                continue
            _, nons = k.split(':')
            if nons in directives:
                ## clean this up by making the names more consistent
                specials[directives[nons]] = directive(v)
                del attrs[k]
            if nons in attributes:
                specials[nons] = v
                del attrs[k]
        el = Tag(name, attributes=dict(attrs), specials=specials)
        self.stack.append(el)
        self.current.append(el)
        self.current = el.children

    def characters(self, ch):
        self.current.append(ch)

    def endElement(self, name):
        me = self.stack.pop()
        if self.stack:
            self.current = self.stack[-1].children
        else:
            self.current = self.document


def parse(fl):
    ## Earlier PyXMLs don't handle non-standard entities (e.g. &copy;) 
    ## correctly. They will either give an error or simply ignore the
    ## entity producing bad output.
    
    if bad_version:
        raise Exception("Please use PyXML later than 0.8.2 or python later than 2.3. Earlier ones are too buggy.")
    
    parser = make_parser()
    parser.setFeature(handler.feature_validation, 0)
    parser.setFeature(handler.feature_namespaces, 0)
    parser.setFeature(handler.feature_external_ges, 0)
    parser.setFeature(handler.feature_external_pes, 0)
    
    s = ToStan()
    parser.setContentHandler(s)
    parser.setEntityResolver(s)
    
    parser.parse(fl)

    return s.document

def parseString(t):
    from cStringIO import StringIO
    return parse(StringIO(t))
