# -*- test-case-name: nevow.test -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from __future__ import generators

import os
import stat
import time
import types
import warnings
from time import time as now
import traceback
import warnings

from twisted.python.reflect import qual
from twisted.python import log

from twisted.web import resource, server, microdom
from twisted.internet.defer import Deferred
from twisted.protocols import http

from nevow import inevow
from nevow.inevow import ISerializable, IRendererFactory, IRequest, IData, IContainer, ICanHandleNotFound
from nevow.stan import xml, sequence, mapping, mergeSlot
from nevow import tags
from nevow.context import WovenContext
from nevow import liveevil
from nevow import compy

# FIXME: hack!
# and the way serialize is implemented is possibly an
# abuse of the compy system, but hey, it WORKS. :)

cachedAdapters = {}
def getSerializer(obj):
    if hasattr(obj, '__class__'):
        klas = obj.__class__
    else:
        klas = type(obj)

    adapter = cachedAdapters.get(klas, None)
    if adapter is not None:
        return adapter
    
    # print "Adding cache entry for ",klas
    registry = compy.getRegistry(None)
    fromInterfaces = compy.classToInterfaces(klas)
    for fromInterface in fromInterfaces:
        # print " trying: ", fromInterface
        adapter = registry.getAdapterFactory(fromInterface, ISerializable, None)
        if adapter is not None:
            cachedAdapters[klas] = adapter
            return adapter
    return None

def serialize(obj, context):
    serializer = getSerializer(obj)
    if serializer is not None:
        return serializer(obj, context)

    raise NotImplementedError('%s instance (type %s) does not implement %s, and '
                              'there is no registered adapter.' %
                              (obj, type(obj), ISerializable))


def _flatten(gen, straccum, results):
    for item in gen:
        if isinstance(item, str):
            straccum.append(item)
        elif isinstance(item, unicode):
            straccum.append(item.encode('utf-8'))
        elif isinstance(item, (types.GeneratorType, list)):
            _flatten(item, straccum, results)
        else:
            # anything that can be serialize()d
            if straccum:
                results.append(xml(''.join(straccum)))
            results.append(item)
            del straccum[:]

def flatten(gen):
    """
    I am a permissive flattener for precompilation.
    """
    straccum = []
    results = []
    _flatten(gen, straccum, results)
    if straccum:
        results.append(xml(''.join(straccum)))
    return results

def _strflatten(context, gen, buf):
    """
    I am a flattener which only allows strings.
    """
    for item in gen:
        if isinstance(item, str):
            buf(item)
        elif isinstance(item, unicode):
            buf(item.encode('utf-8'))
        elif isinstance(item, Deferred):
            yield item
            if not hasattr(item, 'result'):
                print "ERROR: Deferred has no result!"
                buf("ERROR: Deferred has no result!")
            else:
                for sub in _strflatten(context, serialize(item.result, context), buf):
                    yield sub
        else:
            for sub in _strflatten(context, item, buf):
                yield sub

from cStringIO import StringIO
from twisted.python.context import get as getCtx

def _error(failure):
    # FIXME: this should *definitely* do something better
    # doing this leaves the connection hanging, but it's better
    # than nothing.
    print "FAILURE !",failure 
    
def _subrender(L, itr, context, writer, closer):
    for item in itr:
        if writer is not None:
            data = ''.join(L)
            writer(data)
            del L[:]
        cfac = getCtx("CursorFactory")
        if cfac:
            transact = cfac.store.transact
            item.addCallback(lambda ignored: transact(_subrender, L, itr, context, writer, closer))
        else:
            item.addCallback(lambda ignored: _subrender(L, itr, context, writer, closer))
        item.addErrback(_error)
        return item
    data = ''.join(L)
    if writer is not None:
        writer(data)
    if closer is not None:
        closer()
    return data

def render(gen, context=None, writer=None, closer=None):
    L = []
    itr = _strflatten(context, gen, L.append)
    return _subrender(L, itr, context, writer, closer)


class RenderFactory(object):
    __implements__ = IRendererFactory,

    def renderer(self, context, name):
        """Return a renderer with the given name.
        """

        # The named renderer can be parameterised, i.e. 'renderIt one,two,three'
        args = []
        if name.find(' ') != -1:
            name, args = name.split(None, 1)
            args = [arg.strip() for arg in args.split(',')]
        
        callable = getattr(self, 'render_%s' % name, None)
        if callable is None:
            callable = lambda context, data: context.tag["The renderer named '%s' was not found in %r." % (name, self)]

        if args:
            return callable(*args)

        return callable

    render_sequence = lambda self, context, data: sequence(context, data)
    render_mapping = lambda self, context, data: mapping(context, data)
    render_string = lambda self, context, data: context.tag.clear()[str(data)]
    render_xml = lambda self, context, data: context.tag.clear()[xml(data)]
    render_remove = lambda self, context, data: ''
    render_mergeSlot = lambda self, slot: mergeSlot(slot)

class DataFactory(object):
    __implements__ = IContainer,

    def child(self, context, name):
        callable = getattr(self, 'data_%s' % name, None)
        if callable is None:
            container = IContainer(self.original, None)
            if container is None:
                callable = lambda context, data: context.tag["The data named '%s' was not found in %r." % (name, self)]
            else:
                callable = container.child
        return callable


_documents = {}


class ChildPrefixMixin(object):
    def getChild(self, name, request):
        w = getattr(self, 'child_%s' %name, None)
        if w:
            return w(request)
        result = self.getDynamicChild(name, request)
        if result is None:
            return FourOhFour(None)
        return result

    def getDynamicChild(self, name, request):
        pass # 404

    def child_(self, request):
        return self

    def child_nevow_liveOutput(self, request):
        return liveevil.liveOutput

    def child_nevow_liveInput(self, request):
        return liveevil.liveInput


class Renderer(ChildPrefixMixin, resource.Resource, RenderFactory, DataFactory):
    """**** Deprecated ****. Use rend.Page instead.
    Subclass me to implement a resource which uses a stan Tag tree
    to render a web page. Declare an html proto tag instance with children
    as a 'document' property. The document will be precompiled, optimizing
    out static strings, and when render is called, the document will be rendered
    with self as the main 'data' object.

    This class also remembers self as IResource and IRendererFactory, and
    remembers the web request as IRequest. So for example, you may
    locate the request from any object which has access to a context using
    context.locate(IRequest).

    This class also provides Deferred support.  If you want to allow Deferreds
    in your pages, but you have to set headers or do other such work from
    within a Deferred, set the 'buffered' attribute to True.

    If you subclass this, you can override the beforeRender and afterRender
    methods to do any setup/teardown work which you may need to do
    before and after each render.
    """
    original = None
    buffered = False
    _context = None

    __implements__ = RenderFactory.__implements__, DataFactory.__implements__, resource.Resource.__implements__

    def __init__(self, original=None):
        warnings.warn("renderer.Renderer has been deprecated. Use rend.Page instead.",  DeprecationWarning, 2)
        if original is not None:
            self.original = original
        self.doc = self.precompile()
        self.toremember = []
        resource.Resource.__init__(self)

    def remember(self, obj, inter=None):
        """Remember the given object for the given interfaces (or all interfaces
        obj.__implements__) in the top-level context for this page.

        Note that the objects remembered using this API are only remembered
        for the duration of one render.
        """
        if self.context is None:
            self.toremember.append((obj, inter))
        else:
            ## We have a context. Remember this immediately.
            self.context.remember(obj, inter)

    def precompile(self):
        klsnm = qual(self.__class__)
        if klsnm in _documents:
            return _documents[klsnm]
        context = WovenContext(precompile=True)
        context.remember(self, resource.IResource)
        context.remember(self, inevow.IResource)
        context.remember(self, IRendererFactory)
        _documents[klsnm] = rv = flatten(serialize(self.document, context))
        # print "Precompiled:",rv
        return rv

    def getParentContext(self, request):
        contexts = getattr(request, 'contexts', [])
        if contexts:
            return contexts[-1]
        return None

    beforeRender = None
    afterRender = None
    def render(self, request):
        request.setHeader('content-type', "text/html; charset=utf-8")
        if self.beforeRender is not None:
            self.beforeRender(request)
        log.msg(http_render=None, uri=request.uri)
        self.remember(self, resource.IResource)
        self.remember(self, inevow.IResource)
        self.remember(self, IRendererFactory)
        self.remember(request, IRequest)

        if self.original is None:
            self.remember(self, IData)
        else:
            self.remember(self.original, IData)
            self.remember(self.original)

        def finishRequest():
            from nevow import freeform
            sess = request.getSession()
            freeform.IFormErrors(sess).clearAll()
            freeform.IFormDefaults(sess).clearAll()
            self.context = None
            if self.afterRender is not None:
                self.afterRender(request)

        if self.buffered:
            io = StringIO()
            writer = io.write
            def finisher():
                request.write(io.getvalue())
                finishRequest()
                return request.finish()
        else:
            writer = request.write
            def finisher():
                finishRequest()
                return request.finish()
        if self.context is None:
            ## We were created outside of the normal locateChild lookup
            ## loop, and thus don't have any context to which we are attached.
            ## create one.
            self.context = WovenContext(tag=self)
        assert self.context is not None
        render(
            serialize(self.doc, self.context),
            self.context, writer, finisher)
        return server.NOT_DONE_YET

    def getChild(self, name, request):
        ## The method or property name we are going to validate against/affect
        bindingName = None

        if name.startswith('freeform_post!'):
            bindingName = name.split('!')[-1]
        if name.startswith('freeform-action-post!'):
            request.args['freeform-actee'] = name.split('!')[-1]
            bindingName = request.args['freeform-action'][0]
        if bindingName:
            ctx = self.context
            if ctx is None:
                ctx = request.context
            ctx.remember(self, resource.IResource)
            ctx.remember(self, inevow.IResource)
            ctx.remember(request, IRequest)
            from nevow import freeform
            configurable = freeform.locateConfigurable(ctx, getattr(self, 'original', self))
            return self.webFormPost(request, self, configurable, ctx, bindingName, request.args)
        return ChildPrefixMixin.getChild(self, name, request)

    def webFormPost(self, request, res, configurable, ctx, bindingName, args):
        return configurable.postForm(ctx, bindingName, args)

    def _setContext(self, new):
        self._context = new
        if new is not None:
            for rememberit in self.toremember:
                new.remember(*rememberit)

    context = property(
        lambda self: self._context,
        _setContext)

class HTMLRenderer(Renderer):
    """**** Deprecated ****. Use rend.Page instead.
    Subclass me to implement a resource which uses a XHTML template to
    render a web page. The template can be specified as a string (template)
    or as a file on disk (templateFile and optional templateDirectory).
    """
    templateDirectory = ''
    templateFile = ''
    template = None
    key = 'content'
    beExtremelyLenient=False

    def __init__(self, original=None, templateDirectory=None, templateFile=None, template=None, key=None, beExtremelyLenient=None):
        warnings.warn("renderer.HTMLRenderer has been deprecated. Use rend.Page instead.",  DeprecationWarning, 2)
        if templateDirectory is not None:
            self.templateDirectory = templateDirectory
        if templateFile is not None:
            self.templateFile = templateFile
        if template is not None:
            self.template = template
        if key is not None:
            self.key = key
        if beExtremelyLenient is not None:
            self.beExtremelyLenient = beExtremelyLenient
        self.precompileTime = None
        Renderer.__init__(self, original)

    def precompile(self):
        context = WovenContext(precompile=True).remember(self, resource.IResource)
        if self.template is not None:
            dom = microdom.parseString(self.template, beExtremelyLenient=self.beExtremelyLenient)
        else:
            dom = microdom.parse(os.path.join(self.templateDirectory, self.templateFile), beExtremelyLenient=self.beExtremelyLenient)
        doc = flatten(serialize(dom, context))
        # Precompiled. Record the time so we know when to reload the template.
        self.precompileTime = time.time()
        return doc

    def check(self):
        if self.template is not None:
            return
        # Precompile (again) if the template file has been changed
        fullpath = os.path.join(self.templateDirectory, self.templateFile)
        if self.precompileTime is None or \
               os.stat(fullpath)[stat.ST_CTIME] > self.precompileTime:
            self.doc = self.precompile()

    def render(self, request):
        self.check()
        return Renderer.render(self,request)

    def __call__(self, context, data):
        self.check()
        ctx = context.with(tags.invisible[self.doc])
        return ctx.allPatterns(self.key)


class XMLRenderer(HTMLRenderer):
    """**** Deprecated ****. Use rend.Page instead.
    A renderer that uses the builtin python sax to parse the template off disk,
    so your XML betta be well formed, boy
    """
    def precompile(self):
        warnings.warn("renderer.XMLRenderer has been deprecated. Use rend.Page instead.",  DeprecationWarning, 2)
        from nevow.flat import flatsax
        context = WovenContext(precompile=True).remember(self, resource.IResource)
        if self.template is not None:
            dom = flatsax.parse(self.template)
        else:
            dom = flatsax.parse(open(os.path.join(self.templateDirectory, self.templateFile)))
        doc = flatten(serialize(dom, context))
        # Precompiled. Record the time so we know when to reload the template.
        self.precompileTime = time.time()
        return doc

    def render(self, request):
# cannot use text/xml because it breaks MSIE
# TODO: use text/xml when browser sends "Accept" header indicating support.
        # request.setHeader('content-type', 'text/xml')
        return HTMLRenderer.render(self, request)

    ## TODO use a different exception handler because browsers don't like it when you say the
    ## content-type is text/xml and you have nasty html


class DiskTemplateRenderer(HTMLRenderer):
    """**** Deprecated ****. Use rend.Page instead.
    """
    def __init__(self, *args, **kw):
        warnings.warn("DiskTemplateRenderer has been renamed HTMLRenderer.", stacklevel=2)
        HTMLRenderer.__init__(self, *args, **kw)


class DynamicRenderer(Renderer):
    """**** Deprecated ****. Use rend.Page instead.
    Instantiate me to create an IResource implementor which will render
    any object which implements ISerializable. This will usually be a stan Tag
    instance used by calling the html proto and using stan syntax to build a
    dom of child nodes. Use this renderer to create very dynamic pages whose
    template varies every time the page is rendered, or just to get some html
    onto a web page very quickly.

    This subclass of Renderer uses the same rememberances and can be used
    to provide render_ methods and data_ methods to your template in a
    similar way.
    """
    def __init__(self, doc, data=None):
        warnings.warn("renderer.DynamicRenderer has been deprecated. Use rend.Page instead.",  DeprecationWarning, 2)
        self.original = data
        context = WovenContext(precompile=True).remember(self, resource.IResource)
        context.remember(self, IRendererFactory)
        self.doc = flatten(serialize(doc, context))
        ## Bypass Renderer init
        self.toremember = []
        resource.Resource.__init__(self)


class FourOhFour(Renderer):
    isLeaf = True
    document = tags.html[
        tags.head[tags.title["Page Not Found"]],
        tags.body[
            "Sorry, but I couldn't find the object you requested."
        ]
    ]

    def render(self, request):
        request.setResponseCode(http.NOT_FOUND)
        if request.contexts:
            ctx = request.contexts[-1]
        else:
            ctx = request.context
        try:
            ctx.locate(ICanHandleNotFound).renderHTTP_notFound(request)
        except KeyError, e:
            return Renderer.render(self, request)
        except Exception, e:
            traceback.print_exc()
            return Renderer.render(self, request)
                
    def __nonzero__(self):
        return False
